(*
BLAST is a tool for software model checking.
This file is part of BLAST.

Copyright (c) 2002-2008, The BLAST Team.
All rights reserved. 

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.


The BLAST Team consists of
        Dirk Beyer (SFU), Thomas A. Henzinger (EPFL),
        Ranjit Jhala (UCSD), and Rupak Majumdar (UCLA).

BLAST web page:
        http://mtc.epfl.ch/blast/

Bug reports:
        Dirk Beyer:      firstname.lastname@sfu.ca or
        Rupak Majumdar:  firstname@cs.ucla.edu or
        Ranjit Jhala:    lastname@cs.ucla.edu
 *)
type token =
  | Id of (string)
  | Const of (string)
  | Num of (int)
  | GLOBAL
  | LOCAL
  | ENV
  | STORE
  | PROC
  | PREDICATES
  | ANNOTATIONS
  | SEMICOLON
  | COLON
  | EOF
  | LPAREN
  | RPAREN
  | LINDEX
  | RINDEX
  | QUESTION
  | DOT
  | ARROW
  | BITAND
  | BITOR
  | DIV
  | LSHIFT
  | MINUS
  | MUL
  | PLUS
  | REM
  | RSHIFT
  | XOR
  | EQ
  | GE
  | GT
  | LE
  | LT
  | NE
  | INDEX
  | AND
  | OR
  | COMMA
  | NOT
  | BITNOT
  | DEREF
  | ADDRESS
  | SYMBOL
  | UMINUS
  | UPLUS
  | DOTSTAR
  | ARROWSTAR
  | ICR
  | DECR
  | SIZEOF
  | ASSIGN
  | MULTassign
  | DIVassign
  | MODassign
  | PLUSassign
  | MINUSassign
  | LSassign
  | RSassign
  | ANDassign
  | XORassign
  | ORassign
  | INT
  | CHAR
  | FLOAT
  | DOUBLE
  | LONG
  | VOID
  | CONSTANT

open Parsing;;
# 3 "inputparse.mly"
module E = VampyreErrormsg
open Ast

let  parse_error msg =
    E.s (E.error "Parse error at %s: %s" 
           (E.getLineCol !E.current (symbol_start ())) msg)

let predicateList = ref []
let resetPredicateParser () = predicateList := []
let pushPredicate  pred =
  predicateList := pred :: !predicateList
# 87 "inputparse.ml"
let yytransl_const = [|
  260 (* GLOBAL *);
  261 (* LOCAL *);
  262 (* ENV *);
  263 (* STORE *);
  264 (* PROC *);
  265 (* PREDICATES *);
  266 (* ANNOTATIONS *);
  267 (* SEMICOLON *);
  268 (* COLON *);
    0 (* EOF *);
  269 (* LPAREN *);
  270 (* RPAREN *);
  271 (* LINDEX *);
  272 (* RINDEX *);
  273 (* QUESTION *);
  274 (* DOT *);
  275 (* ARROW *);
  276 (* BITAND *);
  277 (* BITOR *);
  278 (* DIV *);
  279 (* LSHIFT *);
  280 (* MINUS *);
  281 (* MUL *);
  282 (* PLUS *);
  283 (* REM *);
  284 (* RSHIFT *);
  285 (* XOR *);
  286 (* EQ *);
  287 (* GE *);
  288 (* GT *);
  289 (* LE *);
  290 (* LT *);
  291 (* NE *);
  292 (* INDEX *);
  293 (* AND *);
  294 (* OR *);
  295 (* COMMA *);
  296 (* NOT *);
  297 (* BITNOT *);
  298 (* DEREF *);
  299 (* ADDRESS *);
  300 (* SYMBOL *);
  301 (* UMINUS *);
  302 (* UPLUS *);
  303 (* DOTSTAR *);
  304 (* ARROWSTAR *);
  305 (* ICR *);
  306 (* DECR *);
  307 (* SIZEOF *);
  308 (* ASSIGN *);
  309 (* MULTassign *);
  310 (* DIVassign *);
  311 (* MODassign *);
  312 (* PLUSassign *);
  313 (* MINUSassign *);
  314 (* LSassign *);
  315 (* RSassign *);
  316 (* ANDassign *);
  317 (* XORassign *);
  318 (* ORassign *);
  319 (* INT *);
  320 (* CHAR *);
  321 (* FLOAT *);
  322 (* DOUBLE *);
  323 (* LONG *);
  324 (* VOID *);
  325 (* CONSTANT *);
    0|]

let yytransl_block = [|
  257 (* Id *);
  258 (* Const *);
  259 (* Num *);
    0|]

let yylhs = "\255\255\
\003\000\003\000\005\000\010\000\010\000\011\000\011\000\012\000\
\012\000\013\000\013\000\001\000\001\000\016\000\002\000\004\000\
\014\000\014\000\017\000\017\000\006\000\009\000\009\000\009\000\
\009\000\009\000\018\000\018\000\018\000\022\000\022\000\022\000\
\022\000\022\000\022\000\022\000\022\000\023\000\023\000\008\000\
\008\000\008\000\008\000\008\000\008\000\008\000\025\000\025\000\
\025\000\025\000\025\000\026\000\026\000\028\000\028\000\028\000\
\028\000\029\000\029\000\029\000\030\000\030\000\030\000\031\000\
\031\000\031\000\031\000\031\000\007\000\007\000\007\000\032\000\
\032\000\033\000\033\000\034\000\034\000\035\000\035\000\036\000\
\036\000\037\000\037\000\024\000\024\000\038\000\038\000\038\000\
\038\000\038\000\038\000\038\000\038\000\038\000\038\000\038\000\
\021\000\021\000\020\000\019\000\015\000\015\000\027\000\027\000\
\000\000\000\000\000\000\000\000\000\000"

let yylen = "\002\000\
\001\000\005\000\005\000\001\000\005\000\001\000\005\000\001\000\
\005\000\001\000\013\000\000\000\003\000\003\000\002\000\002\000\
\000\000\003\000\000\000\003\000\001\000\001\000\003\000\003\000\
\002\000\003\000\001\000\001\000\003\000\001\000\004\000\003\000\
\004\000\003\000\003\000\002\000\002\000\001\000\003\000\001\000\
\002\000\002\000\002\000\002\000\002\000\004\000\001\000\001\000\
\001\000\001\000\001\000\001\000\004\000\001\000\003\000\003\000\
\003\000\001\000\003\000\003\000\001\000\003\000\003\000\001\000\
\003\000\003\000\003\000\003\000\001\000\003\000\003\000\001\000\
\003\000\001\000\003\000\001\000\003\000\001\000\003\000\001\000\
\003\000\001\000\005\000\001\000\003\000\001\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\003\000\001\000\001\000\000\000\003\000\001\000\001\000\
\002\000\002\000\002\000\002\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\105\000\
\000\000\100\000\099\000\000\000\047\000\049\000\000\000\048\000\
\051\000\050\000\000\000\000\000\000\000\106\000\000\000\000\000\
\052\000\000\000\030\000\027\000\028\000\000\000\000\000\054\000\
\000\000\000\000\000\000\000\000\001\000\000\000\107\000\000\000\
\108\000\000\000\000\000\000\000\004\000\109\000\000\000\000\000\
\000\000\103\000\104\000\000\000\000\000\000\000\097\000\000\000\
\000\000\000\000\000\000\000\000\000\000\084\000\044\000\041\000\
\042\000\000\000\045\000\000\000\000\000\000\000\015\000\000\000\
\000\000\000\000\000\000\036\000\037\000\043\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\016\000\000\000\000\000\006\000\000\000\
\014\000\013\000\086\000\087\000\088\000\089\000\090\000\091\000\
\092\000\093\000\094\000\095\000\096\000\000\000\029\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\020\000\000\000\000\000\032\000\000\000\038\000\000\000\034\000\
\035\000\056\000\055\000\057\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\022\000\000\000\
\018\000\000\000\000\000\008\000\000\000\000\000\085\000\098\000\
\053\000\000\000\000\000\000\000\000\000\000\000\000\000\046\000\
\033\000\000\000\031\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\010\000\000\000\000\000\
\039\000\026\000\002\000\000\000\000\000\005\000\000\000\000\000\
\000\000\000\000\003\000\083\000\007\000\000\000\000\000\000\000\
\102\000\009\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\011\000"

let yydgoto = "\006\000\
\008\000\022\000\039\000\041\000\046\000\143\000\052\000\025\000\
\144\000\047\000\096\000\150\000\175\000\043\000\185\000\009\000\
\026\000\027\000\028\000\029\000\054\000\030\000\125\000\055\000\
\031\000\032\000\056\000\033\000\034\000\035\000\036\000\057\000\
\058\000\059\000\060\000\061\000\062\000\110\000"

let yysindex = "\136\000\
\019\255\227\002\001\000\003\003\013\255\000\000\020\255\000\000\
\016\255\000\000\000\000\041\255\000\000\000\000\227\002\000\000\
\000\000\000\000\003\003\003\003\006\003\000\000\038\255\000\255\
\000\000\055\000\000\000\000\000\000\000\044\255\227\002\000\000\
\091\255\125\255\018\255\111\255\000\000\227\002\000\000\046\255\
\000\000\053\255\068\000\058\255\000\000\000\000\022\255\227\002\
\019\255\000\000\000\000\000\255\027\001\239\255\000\000\065\255\
\069\255\078\255\093\255\087\255\251\254\000\000\000\000\000\000\
\000\000\227\002\000\000\227\002\227\002\227\002\000\000\192\002\
\227\002\149\255\149\255\000\000\000\000\000\000\227\002\227\002\
\227\002\227\002\227\002\227\002\227\002\227\002\227\002\227\002\
\227\002\038\003\003\003\000\000\003\003\143\255\000\000\195\002\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\227\002\000\000\227\002\
\227\002\227\002\227\002\227\002\227\002\227\002\227\002\155\255\
\000\000\111\255\111\255\000\000\253\254\000\000\215\255\000\000\
\000\000\000\000\000\000\000\000\091\255\091\255\125\255\125\255\
\018\255\018\255\018\255\018\255\107\255\038\003\000\000\084\255\
\000\000\134\255\003\003\000\000\162\255\070\255\000\000\000\000\
\000\000\000\255\069\255\078\255\093\255\218\255\087\255\000\000\
\000\000\227\002\000\000\000\255\015\255\092\255\001\000\038\003\
\038\003\013\255\136\255\175\255\174\255\000\000\178\000\227\002\
\000\000\000\000\000\000\092\255\092\255\000\000\022\255\140\255\
\144\255\145\255\000\000\000\000\000\000\175\255\195\002\179\255\
\000\000\000\000\146\255\183\255\148\255\186\255\176\255\180\255\
\003\003\152\255\070\255\000\000"

let yyrindex = "\000\000\
\193\000\194\000\000\000\195\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\037\255\
\000\000\000\000\000\000\000\000\000\000\092\000\000\000\000\000\
\144\000\234\000\084\001\232\001\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\181\255\
\193\000\000\000\000\000\251\255\212\255\000\000\000\000\000\000\
\058\002\093\002\134\002\124\000\240\255\000\000\000\000\000\000\
\000\000\000\000\000\000\002\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\004\000\000\000\158\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\040\000\
\000\000\008\002\019\002\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\173\000\204\000\010\001\039\001\
\113\001\142\001\173\001\203\001\000\000\000\000\000\000\000\000\
\000\000\000\000\158\255\000\000\000\000\000\000\000\000\000\000\
\000\000\047\002\087\002\121\002\151\002\000\000\105\002\000\000\
\000\000\000\000\000\000\165\002\000\000\008\255\000\000\000\000\
\000\000\000\000\000\000\159\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\095\255\114\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\159\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\151\000\000\000\034\000\000\000\000\000\008\000\012\000\005\000\
\187\255\032\000\020\000\013\000\007\000\178\255\015\000\000\000\
\226\255\000\000\037\000\000\000\188\255\000\000\000\000\191\255\
\000\000\006\000\000\000\052\000\079\000\073\000\096\000\093\000\
\091\000\094\000\090\000\000\000\036\000\000\000"

let yytablesize = 1113
let yytable = "\052\000\
\037\000\019\000\112\000\017\000\127\000\112\000\126\000\040\000\
\042\000\023\000\161\000\118\000\145\000\024\000\146\000\044\000\
\053\000\097\000\025\000\007\000\063\000\025\000\094\000\064\000\
\065\000\067\000\112\000\082\000\178\000\069\000\045\000\048\000\
\119\000\049\000\070\000\162\000\078\000\121\000\072\000\095\000\
\084\000\010\000\053\000\011\000\151\000\085\000\152\000\021\000\
\068\000\158\000\021\000\168\000\169\000\012\000\071\000\023\000\
\072\000\090\000\073\000\024\000\013\000\074\000\075\000\091\000\
\014\000\015\000\016\000\092\000\171\000\093\000\053\000\165\000\
\166\000\021\000\021\000\023\000\053\000\053\000\113\000\024\000\
\017\000\018\000\173\000\027\000\130\000\131\000\132\000\174\000\
\114\000\019\000\020\000\021\000\076\000\077\000\167\000\042\000\
\177\000\042\000\180\000\181\000\149\000\024\000\120\000\050\000\
\051\000\023\000\115\000\010\000\023\000\011\000\128\000\129\000\
\079\000\116\000\053\000\080\000\053\000\081\000\153\000\141\000\
\168\000\169\000\053\000\117\000\024\000\154\000\013\000\024\000\
\168\000\169\000\014\000\015\000\016\000\133\000\134\000\040\000\
\001\000\002\000\003\000\004\000\005\000\086\000\087\000\088\000\
\089\000\053\000\142\000\018\000\082\000\010\000\083\000\042\000\
\164\000\024\000\147\000\019\000\020\000\021\000\137\000\138\000\
\139\000\140\000\135\000\136\000\122\000\123\000\053\000\080\000\
\160\000\050\000\051\000\040\000\170\000\172\000\183\000\184\000\
\186\000\187\000\190\000\024\000\024\000\195\000\191\000\192\000\
\196\000\197\000\198\000\058\000\199\000\200\000\203\000\201\000\
\012\000\019\000\017\000\149\000\017\000\101\000\019\000\098\000\
\179\000\182\000\189\000\194\000\193\000\202\000\156\000\155\000\
\159\000\204\000\157\000\188\000\000\000\000\000\000\000\000\000\
\060\000\000\000\000\000\000\000\000\000\000\000\000\000\052\000\
\000\000\052\000\000\000\052\000\052\000\176\000\163\000\052\000\
\052\000\052\000\052\000\052\000\052\000\052\000\052\000\052\000\
\052\000\052\000\052\000\052\000\052\000\052\000\052\000\059\000\
\052\000\052\000\052\000\082\000\111\000\082\000\000\000\082\000\
\000\000\010\000\000\000\011\000\000\000\000\000\072\000\000\000\
\072\000\000\000\072\000\072\000\000\000\038\000\072\000\072\000\
\000\000\000\000\000\000\019\000\013\000\061\000\082\000\072\000\
\014\000\015\000\016\000\000\000\000\000\000\000\000\000\072\000\
\072\000\072\000\000\000\000\000\000\000\000\000\000\000\000\000\
\017\000\018\000\017\000\000\000\000\000\000\000\000\000\000\000\
\000\000\019\000\020\000\021\000\027\000\062\000\027\000\000\000\
\027\000\027\000\027\000\027\000\027\000\027\000\027\000\027\000\
\027\000\027\000\027\000\027\000\027\000\027\000\027\000\027\000\
\027\000\027\000\027\000\000\000\027\000\027\000\000\000\000\000\
\000\000\000\000\063\000\000\000\000\000\000\000\000\000\000\000\
\027\000\027\000\000\000\027\000\027\000\027\000\027\000\027\000\
\027\000\027\000\027\000\027\000\027\000\027\000\040\000\040\000\
\000\000\040\000\000\000\040\000\040\000\000\000\000\000\040\000\
\040\000\040\000\040\000\040\000\040\000\040\000\040\000\040\000\
\040\000\040\000\040\000\040\000\040\000\040\000\040\000\064\000\
\040\000\040\000\040\000\000\000\000\000\000\000\000\000\080\000\
\000\000\080\000\000\000\080\000\080\000\000\000\000\000\040\000\
\040\000\040\000\040\000\040\000\040\000\040\000\040\000\040\000\
\040\000\040\000\058\000\058\000\068\000\058\000\000\000\058\000\
\058\000\080\000\080\000\058\000\058\000\000\000\058\000\058\000\
\000\000\058\000\000\000\058\000\058\000\058\000\058\000\058\000\
\058\000\058\000\058\000\000\000\058\000\058\000\058\000\060\000\
\060\000\066\000\060\000\000\000\060\000\060\000\000\000\000\000\
\060\000\060\000\000\000\060\000\060\000\000\000\060\000\000\000\
\060\000\060\000\060\000\060\000\060\000\060\000\060\000\060\000\
\000\000\060\000\060\000\060\000\000\000\000\000\059\000\059\000\
\067\000\059\000\000\000\059\000\059\000\000\000\000\000\059\000\
\059\000\000\000\059\000\059\000\000\000\059\000\000\000\059\000\
\059\000\059\000\059\000\059\000\059\000\059\000\059\000\000\000\
\059\000\059\000\059\000\000\000\061\000\061\000\065\000\061\000\
\000\000\061\000\061\000\000\000\000\000\061\000\061\000\000\000\
\061\000\000\000\000\000\000\000\000\000\061\000\061\000\061\000\
\061\000\061\000\061\000\061\000\061\000\000\000\061\000\061\000\
\061\000\000\000\000\000\069\000\062\000\062\000\000\000\062\000\
\000\000\062\000\062\000\000\000\000\000\062\000\062\000\000\000\
\062\000\000\000\000\000\000\000\000\000\062\000\062\000\062\000\
\062\000\062\000\062\000\062\000\062\000\000\000\062\000\062\000\
\062\000\063\000\063\000\070\000\063\000\000\000\063\000\063\000\
\000\000\000\000\063\000\063\000\000\000\063\000\071\000\000\000\
\000\000\000\000\063\000\063\000\063\000\063\000\063\000\063\000\
\063\000\063\000\000\000\063\000\063\000\063\000\099\000\100\000\
\101\000\102\000\103\000\104\000\105\000\106\000\107\000\108\000\
\109\000\000\000\073\000\000\000\000\000\000\000\064\000\064\000\
\000\000\064\000\000\000\064\000\064\000\074\000\000\000\064\000\
\064\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\064\000\064\000\064\000\064\000\064\000\064\000\064\000\000\000\
\064\000\064\000\064\000\068\000\068\000\000\000\068\000\000\000\
\068\000\068\000\075\000\000\000\068\000\068\000\000\000\000\000\
\076\000\000\000\000\000\000\000\000\000\068\000\068\000\068\000\
\068\000\068\000\068\000\068\000\081\000\068\000\068\000\068\000\
\066\000\066\000\000\000\066\000\000\000\066\000\066\000\000\000\
\000\000\066\000\066\000\000\000\077\000\000\000\000\000\000\000\
\000\000\000\000\066\000\066\000\066\000\066\000\066\000\066\000\
\066\000\078\000\066\000\066\000\066\000\000\000\000\000\067\000\
\067\000\000\000\067\000\000\000\067\000\067\000\000\000\000\000\
\067\000\067\000\079\000\000\000\000\000\000\000\000\000\000\000\
\000\000\067\000\067\000\067\000\067\000\067\000\067\000\067\000\
\072\000\067\000\067\000\067\000\000\000\065\000\065\000\000\000\
\065\000\000\000\065\000\065\000\000\000\000\000\065\000\065\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\065\000\
\065\000\065\000\065\000\065\000\065\000\065\000\000\000\065\000\
\065\000\065\000\069\000\069\000\000\000\069\000\000\000\069\000\
\069\000\000\000\000\000\069\000\069\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\069\000\069\000\000\000\000\000\
\000\000\000\000\069\000\000\000\069\000\069\000\069\000\000\000\
\000\000\000\000\070\000\070\000\000\000\070\000\000\000\070\000\
\070\000\000\000\000\000\070\000\070\000\071\000\071\000\000\000\
\071\000\000\000\071\000\071\000\070\000\070\000\071\000\071\000\
\000\000\000\000\070\000\000\000\070\000\070\000\070\000\071\000\
\071\000\000\000\000\000\000\000\000\000\071\000\000\000\071\000\
\071\000\071\000\073\000\000\000\073\000\000\000\073\000\073\000\
\000\000\000\000\073\000\073\000\000\000\074\000\000\000\074\000\
\000\000\074\000\074\000\073\000\000\000\000\000\074\000\000\000\
\000\000\000\000\000\000\073\000\073\000\073\000\074\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\074\000\074\000\
\074\000\000\000\075\000\000\000\075\000\000\000\075\000\075\000\
\076\000\000\000\076\000\075\000\076\000\076\000\000\000\000\000\
\000\000\076\000\000\000\075\000\081\000\000\000\081\000\000\000\
\081\000\081\000\000\000\075\000\075\000\075\000\000\000\000\000\
\000\000\076\000\076\000\076\000\077\000\000\000\077\000\000\000\
\077\000\077\000\000\000\000\000\000\000\077\000\081\000\081\000\
\000\000\078\000\000\000\078\000\000\000\078\000\078\000\000\000\
\000\000\000\000\000\000\000\000\000\000\077\000\077\000\077\000\
\000\000\000\000\079\000\000\000\079\000\000\000\079\000\079\000\
\000\000\000\000\078\000\078\000\078\000\000\000\000\000\000\000\
\000\000\000\000\021\000\000\000\000\000\072\000\000\000\000\000\
\072\000\072\000\000\000\079\000\079\000\079\000\000\000\000\000\
\010\000\072\000\011\000\010\000\000\000\011\000\000\000\000\000\
\000\000\021\000\021\000\000\000\012\000\124\000\000\000\038\000\
\000\000\000\000\000\000\013\000\148\000\000\000\013\000\014\000\
\015\000\016\000\014\000\015\000\016\000\000\000\000\000\000\000\
\000\000\000\000\000\000\010\000\000\000\011\000\000\000\017\000\
\018\000\000\000\017\000\018\000\000\000\000\000\000\000\012\000\
\019\000\020\000\021\000\019\000\020\000\021\000\013\000\000\000\
\000\000\000\000\014\000\015\000\016\000\000\000\000\000\000\000\
\000\000\000\000\000\000\010\000\000\000\011\000\010\000\000\000\
\011\000\000\000\017\000\018\000\000\000\000\000\000\000\038\000\
\000\000\000\000\066\000\019\000\020\000\021\000\013\000\000\000\
\000\000\013\000\014\000\015\000\016\000\014\000\015\000\016\000\
\000\000\000\000\000\000\000\000\000\000\000\000\010\000\000\000\
\011\000\000\000\017\000\018\000\000\000\017\000\018\000\000\000\
\000\000\000\000\141\000\019\000\020\000\021\000\019\000\020\000\
\021\000\013\000\000\000\000\000\000\000\014\000\015\000\016\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\142\000\018\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\019\000\020\000\
\021\000"

let yycheck = "\044\000\
\000\000\000\000\044\000\000\000\073\000\044\000\072\000\003\000\
\004\000\002\000\014\001\017\001\091\000\002\000\093\000\003\001\
\012\000\048\000\011\001\001\001\015\000\014\001\001\001\019\000\
\020\000\021\000\044\000\044\000\014\001\030\001\018\001\012\001\
\038\001\018\001\035\001\039\001\031\000\068\000\044\000\018\001\
\023\001\001\001\038\000\003\001\110\000\028\001\112\000\011\001\
\011\001\118\000\014\001\037\001\038\001\013\001\000\000\048\000\
\013\001\012\001\015\001\048\000\020\001\018\001\019\001\011\001\
\024\001\025\001\026\001\000\000\147\000\012\001\066\000\141\000\
\142\000\037\001\038\001\068\000\072\000\073\000\014\001\068\000\
\040\001\041\001\013\001\044\000\079\000\080\000\081\000\018\001\
\020\001\049\001\050\001\051\001\049\001\050\001\011\001\091\000\
\162\000\093\000\168\000\169\000\096\000\090\000\066\000\063\001\
\064\001\011\001\029\001\001\001\014\001\003\001\074\000\075\000\
\022\001\021\001\110\000\025\001\112\000\027\001\113\000\013\001\
\037\001\038\001\118\000\037\001\011\001\114\000\020\001\014\001\
\037\001\038\001\024\001\025\001\026\001\082\000\083\000\044\000\
\001\000\002\000\003\000\004\000\005\000\031\001\032\001\033\001\
\034\001\141\000\040\001\041\001\024\001\001\001\026\001\147\000\
\141\000\142\000\012\001\049\001\050\001\051\001\086\000\087\000\
\088\000\089\000\084\000\085\000\069\000\070\000\162\000\044\000\
\014\001\063\001\064\001\167\000\039\001\012\001\039\001\001\001\
\003\001\000\000\039\001\168\000\169\000\003\001\039\001\039\001\
\039\001\003\001\039\001\044\000\003\001\014\001\039\001\012\001\
\000\000\000\000\000\000\191\000\039\001\039\001\018\001\049\000\
\167\000\170\000\183\000\191\000\190\000\201\000\116\000\115\000\
\119\000\203\000\117\000\176\000\255\255\255\255\255\255\255\255\
\044\000\255\255\255\255\255\255\255\255\255\255\255\255\012\001\
\255\255\014\001\255\255\016\001\017\001\012\001\016\001\020\001\
\021\001\022\001\023\001\024\001\025\001\026\001\027\001\028\001\
\029\001\030\001\031\001\032\001\033\001\034\001\035\001\044\000\
\037\001\038\001\039\001\012\001\014\001\014\001\255\255\016\001\
\255\255\001\001\255\255\003\001\255\255\255\255\012\001\255\255\
\014\001\255\255\016\001\017\001\255\255\013\001\020\001\021\001\
\255\255\255\255\255\255\018\001\020\001\044\000\039\001\029\001\
\024\001\025\001\026\001\255\255\255\255\255\255\255\255\037\001\
\038\001\039\001\255\255\255\255\255\255\255\255\255\255\255\255\
\040\001\041\001\039\001\255\255\255\255\255\255\255\255\255\255\
\255\255\049\001\050\001\051\001\013\001\044\000\015\001\255\255\
\017\001\018\001\019\001\020\001\021\001\022\001\023\001\024\001\
\025\001\026\001\027\001\028\001\029\001\030\001\031\001\032\001\
\033\001\034\001\035\001\255\255\037\001\038\001\255\255\255\255\
\255\255\255\255\044\000\255\255\255\255\255\255\255\255\255\255\
\049\001\050\001\255\255\052\001\053\001\054\001\055\001\056\001\
\057\001\058\001\059\001\060\001\061\001\062\001\011\001\012\001\
\255\255\014\001\255\255\016\001\017\001\255\255\255\255\020\001\
\021\001\022\001\023\001\024\001\025\001\026\001\027\001\028\001\
\029\001\030\001\031\001\032\001\033\001\034\001\035\001\044\000\
\037\001\038\001\039\001\255\255\255\255\255\255\255\255\012\001\
\255\255\014\001\255\255\016\001\017\001\255\255\255\255\052\001\
\053\001\054\001\055\001\056\001\057\001\058\001\059\001\060\001\
\061\001\062\001\011\001\012\001\044\000\014\001\255\255\016\001\
\017\001\038\001\039\001\020\001\021\001\255\255\023\001\024\001\
\255\255\026\001\255\255\028\001\029\001\030\001\031\001\032\001\
\033\001\034\001\035\001\255\255\037\001\038\001\039\001\011\001\
\012\001\044\000\014\001\255\255\016\001\017\001\255\255\255\255\
\020\001\021\001\255\255\023\001\024\001\255\255\026\001\255\255\
\028\001\029\001\030\001\031\001\032\001\033\001\034\001\035\001\
\255\255\037\001\038\001\039\001\255\255\255\255\011\001\012\001\
\044\000\014\001\255\255\016\001\017\001\255\255\255\255\020\001\
\021\001\255\255\023\001\024\001\255\255\026\001\255\255\028\001\
\029\001\030\001\031\001\032\001\033\001\034\001\035\001\255\255\
\037\001\038\001\039\001\255\255\011\001\012\001\044\000\014\001\
\255\255\016\001\017\001\255\255\255\255\020\001\021\001\255\255\
\023\001\255\255\255\255\255\255\255\255\028\001\029\001\030\001\
\031\001\032\001\033\001\034\001\035\001\255\255\037\001\038\001\
\039\001\255\255\255\255\044\000\011\001\012\001\255\255\014\001\
\255\255\016\001\017\001\255\255\255\255\020\001\021\001\255\255\
\023\001\255\255\255\255\255\255\255\255\028\001\029\001\030\001\
\031\001\032\001\033\001\034\001\035\001\255\255\037\001\038\001\
\039\001\011\001\012\001\044\000\014\001\255\255\016\001\017\001\
\255\255\255\255\020\001\021\001\255\255\023\001\044\000\255\255\
\255\255\255\255\028\001\029\001\030\001\031\001\032\001\033\001\
\034\001\035\001\255\255\037\001\038\001\039\001\052\001\053\001\
\054\001\055\001\056\001\057\001\058\001\059\001\060\001\061\001\
\062\001\255\255\044\000\255\255\255\255\255\255\011\001\012\001\
\255\255\014\001\255\255\016\001\017\001\044\000\255\255\020\001\
\021\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\029\001\030\001\031\001\032\001\033\001\034\001\035\001\255\255\
\037\001\038\001\039\001\011\001\012\001\255\255\014\001\255\255\
\016\001\017\001\044\000\255\255\020\001\021\001\255\255\255\255\
\044\000\255\255\255\255\255\255\255\255\029\001\030\001\031\001\
\032\001\033\001\034\001\035\001\044\000\037\001\038\001\039\001\
\011\001\012\001\255\255\014\001\255\255\016\001\017\001\255\255\
\255\255\020\001\021\001\255\255\044\000\255\255\255\255\255\255\
\255\255\255\255\029\001\030\001\031\001\032\001\033\001\034\001\
\035\001\044\000\037\001\038\001\039\001\255\255\255\255\011\001\
\012\001\255\255\014\001\255\255\016\001\017\001\255\255\255\255\
\020\001\021\001\044\000\255\255\255\255\255\255\255\255\255\255\
\255\255\029\001\030\001\031\001\032\001\033\001\034\001\035\001\
\044\000\037\001\038\001\039\001\255\255\011\001\012\001\255\255\
\014\001\255\255\016\001\017\001\255\255\255\255\020\001\021\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\029\001\
\030\001\031\001\032\001\033\001\034\001\035\001\255\255\037\001\
\038\001\039\001\011\001\012\001\255\255\014\001\255\255\016\001\
\017\001\255\255\255\255\020\001\021\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\029\001\030\001\255\255\255\255\
\255\255\255\255\035\001\255\255\037\001\038\001\039\001\255\255\
\255\255\255\255\011\001\012\001\255\255\014\001\255\255\016\001\
\017\001\255\255\255\255\020\001\021\001\011\001\012\001\255\255\
\014\001\255\255\016\001\017\001\029\001\030\001\020\001\021\001\
\255\255\255\255\035\001\255\255\037\001\038\001\039\001\029\001\
\030\001\255\255\255\255\255\255\255\255\035\001\255\255\037\001\
\038\001\039\001\012\001\255\255\014\001\255\255\016\001\017\001\
\255\255\255\255\020\001\021\001\255\255\012\001\255\255\014\001\
\255\255\016\001\017\001\029\001\255\255\255\255\021\001\255\255\
\255\255\255\255\255\255\037\001\038\001\039\001\029\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\037\001\038\001\
\039\001\255\255\012\001\255\255\014\001\255\255\016\001\017\001\
\012\001\255\255\014\001\021\001\016\001\017\001\255\255\255\255\
\255\255\021\001\255\255\029\001\012\001\255\255\014\001\255\255\
\016\001\017\001\255\255\037\001\038\001\039\001\255\255\255\255\
\255\255\037\001\038\001\039\001\012\001\255\255\014\001\255\255\
\016\001\017\001\255\255\255\255\255\255\021\001\038\001\039\001\
\255\255\012\001\255\255\014\001\255\255\016\001\017\001\255\255\
\255\255\255\255\255\255\255\255\255\255\037\001\038\001\039\001\
\255\255\255\255\012\001\255\255\014\001\255\255\016\001\017\001\
\255\255\255\255\037\001\038\001\039\001\255\255\255\255\255\255\
\255\255\255\255\014\001\255\255\255\255\017\001\255\255\255\255\
\020\001\021\001\255\255\037\001\038\001\039\001\255\255\255\255\
\001\001\029\001\003\001\001\001\255\255\003\001\255\255\255\255\
\255\255\037\001\038\001\255\255\013\001\014\001\255\255\013\001\
\255\255\255\255\255\255\020\001\018\001\255\255\020\001\024\001\
\025\001\026\001\024\001\025\001\026\001\255\255\255\255\255\255\
\255\255\255\255\255\255\001\001\255\255\003\001\255\255\040\001\
\041\001\255\255\040\001\041\001\255\255\255\255\255\255\013\001\
\049\001\050\001\051\001\049\001\050\001\051\001\020\001\255\255\
\255\255\255\255\024\001\025\001\026\001\255\255\255\255\255\255\
\255\255\255\255\255\255\001\001\255\255\003\001\001\001\255\255\
\003\001\255\255\040\001\041\001\255\255\255\255\255\255\013\001\
\255\255\255\255\013\001\049\001\050\001\051\001\020\001\255\255\
\255\255\020\001\024\001\025\001\026\001\024\001\025\001\026\001\
\255\255\255\255\255\255\255\255\255\255\255\255\001\001\255\255\
\003\001\255\255\040\001\041\001\255\255\040\001\041\001\255\255\
\255\255\255\255\013\001\049\001\050\001\051\001\049\001\050\001\
\051\001\020\001\255\255\255\255\255\255\024\001\025\001\026\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\040\001\041\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\049\001\050\001\
\051\001"

let yynames_const = "\
  GLOBAL\000\
  LOCAL\000\
  ENV\000\
  STORE\000\
  PROC\000\
  PREDICATES\000\
  ANNOTATIONS\000\
  SEMICOLON\000\
  COLON\000\
  EOF\000\
  LPAREN\000\
  RPAREN\000\
  LINDEX\000\
  RINDEX\000\
  QUESTION\000\
  DOT\000\
  ARROW\000\
  BITAND\000\
  BITOR\000\
  DIV\000\
  LSHIFT\000\
  MINUS\000\
  MUL\000\
  PLUS\000\
  REM\000\
  RSHIFT\000\
  XOR\000\
  EQ\000\
  GE\000\
  GT\000\
  LE\000\
  LT\000\
  NE\000\
  INDEX\000\
  AND\000\
  OR\000\
  COMMA\000\
  NOT\000\
  BITNOT\000\
  DEREF\000\
  ADDRESS\000\
  SYMBOL\000\
  UMINUS\000\
  UPLUS\000\
  DOTSTAR\000\
  ARROWSTAR\000\
  ICR\000\
  DECR\000\
  SIZEOF\000\
  ASSIGN\000\
  MULTassign\000\
  DIVassign\000\
  MODassign\000\
  PLUSassign\000\
  MINUSassign\000\
  LSassign\000\
  RSassign\000\
  ANDassign\000\
  XORassign\000\
  ORassign\000\
  INT\000\
  CHAR\000\
  FLOAT\000\
  DOUBLE\000\
  LONG\000\
  VOID\000\
  CONSTANT\000\
  "

let yynames_block = "\
  Id\000\
  Const\000\
  Num\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun parser_env ->
    Obj.repr(
# 51 "inputparse.mly"
      ( [] )
# 661 "inputparse.ml"
               : ((Ast.Expression.lval * (Ast.Predicate.predicate)) list)))
; (fun parser_env ->
    let _1 = (peek_val parser_env 4 : 'unary_expression) in
    let _3 = (peek_val parser_env 2 : 'assumption) in
    let _5 = (peek_val parser_env 0 : ((Ast.Expression.lval * (Ast.Predicate.predicate)) list)) in
    Obj.repr(
# 53 "inputparse.mly"
     ( 
       match _1 with
	   Expression.Lval l -> (l,_3) :: _5
	 | _ -> failwith "Inputparse : lhs of env_asm is not an lvalue !"
     )
# 674 "inputparse.ml"
               : ((Ast.Expression.lval * (Ast.Predicate.predicate)) list)))
; (fun parser_env ->
    let _1 = (peek_val parser_env 4 : 'int_to_lval_list) in
    let _2 = (peek_val parser_env 3 : 'string_to_lval_list) in
    let _3 = (peek_val parser_env 2 : 'lval_to_string_list) in
    let _4 = (peek_val parser_env 1 : 'four_int_to_lval_list) in
    Obj.repr(
# 64 "inputparse.mly"
                            ( (_1, _2, _3, _4) )
# 684 "inputparse.ml"
               : ((int * (Ast.Expression.lval list)) list ) * ((string * (Ast.Expression.lval list)) list) * ((Ast.Expression.lval * (string list)) list) * (( (int * int * int * int) * Ast.Expression.lval ) list)))
; (fun parser_env ->
    Obj.repr(
# 66 "inputparse.mly"
                       ( [] )
# 690 "inputparse.ml"
               : 'int_to_lval_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 4 : int) in
    let _3 = (peek_val parser_env 2 : 'list_of_lvals) in
    let _5 = (peek_val parser_env 0 : 'int_to_lval_list) in
    Obj.repr(
# 67 "inputparse.mly"
                                                 ( (_1, _3) :: _5 )
# 699 "inputparse.ml"
               : 'int_to_lval_list))
; (fun parser_env ->
    Obj.repr(
# 69 "inputparse.mly"
                          ( [] )
# 705 "inputparse.ml"
               : 'string_to_lval_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 4 : string) in
    let _3 = (peek_val parser_env 2 : 'list_of_lvals) in
    let _5 = (peek_val parser_env 0 : 'string_to_lval_list) in
    Obj.repr(
# 70 "inputparse.mly"
                                                   ( (_1, _3) :: _5 )
# 714 "inputparse.ml"
               : 'string_to_lval_list))
; (fun parser_env ->
    Obj.repr(
# 72 "inputparse.mly"
                          ( [] )
# 720 "inputparse.ml"
               : 'lval_to_string_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 4 : 'unary_expression) in
    let _3 = (peek_val parser_env 2 : 'string_list) in
    let _5 = (peek_val parser_env 0 : 'lval_to_string_list) in
    Obj.repr(
# 74 "inputparse.mly"
   ( match _1 with Expression.Lval l -> (l, _3) :: _5
     | _ -> failwith ("Inputparse: lhs of lval_to_string_list is not an lvalue")
   )
# 731 "inputparse.ml"
               : 'lval_to_string_list))
; (fun parser_env ->
    Obj.repr(
# 78 "inputparse.mly"
                            ( [] )
# 737 "inputparse.ml"
               : 'four_int_to_lval_list))
; (fun parser_env ->
    let _2 = (peek_val parser_env 11 : int) in
    let _4 = (peek_val parser_env 9 : int) in
    let _6 = (peek_val parser_env 7 : int) in
    let _8 = (peek_val parser_env 5 : int) in
    let _11 = (peek_val parser_env 2 : 'unary_expression) in
    let _13 = (peek_val parser_env 0 : 'four_int_to_lval_list) in
    Obj.repr(
# 80 "inputparse.mly"
  ( ((_2, _4, _6, _8), match _11 with Expression.Lval l -> l | _ -> failwith "Inputparse: four_int_to_lval_list: Not an lvalue") :: _13 )
# 749 "inputparse.ml"
               : 'four_int_to_lval_list))
; (fun parser_env ->
    Obj.repr(
# 82 "inputparse.mly"
                            ([])
# 755 "inputparse.ml"
               : ((string * Ast.Predicate.predicate list) list)))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'fun_predicates) in
    let _3 = (peek_val parser_env 0 : ((string * Ast.Predicate.predicate list) list)) in
    Obj.repr(
# 83 "inputparse.mly"
                                (_1::_3)
# 763 "inputparse.ml"
               : ((string * Ast.Predicate.predicate list) list)))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : string) in
    let _3 = (peek_val parser_env 0 : 'predicates) in
    Obj.repr(
# 86 "inputparse.mly"
                                     ( let x = !predicateList in (resetPredicateParser (); (_1,x)) )
# 771 "inputparse.ml"
               : 'fun_predicates))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'predicates) in
    Obj.repr(
# 90 "inputparse.mly"
                   ( (let x = !predicateList in let _ = (predicateList := []) in x ))
# 778 "inputparse.ml"
               : (Ast.Predicate.predicate list)))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'list_of_lvals) in
    Obj.repr(
# 93 "inputparse.mly"
                             ( _1 )
# 785 "inputparse.ml"
               : (Ast.Expression.lval list)))
; (fun parser_env ->
    Obj.repr(
# 96 "inputparse.mly"
   ( [] )
# 791 "inputparse.ml"
               : 'list_of_lvals))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'unary_expression) in
    let _3 = (peek_val parser_env 0 : 'list_of_lvals) in
    Obj.repr(
# 98 "inputparse.mly"
 ( match _1 with
		Expression.Lval l -> (l) :: _3
	  |     _                 -> failwith ("Inputparse: Expected lvalues, got "^(Expression.toString _1))
	)
# 802 "inputparse.ml"
               : 'list_of_lvals))
; (fun parser_env ->
    Obj.repr(
# 106 "inputparse.mly"
  ()
# 808 "inputparse.ml"
               : 'predicates))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : Ast.Predicate.predicate) in
    let _3 = (peek_val parser_env 0 : 'predicates) in
    Obj.repr(
# 107 "inputparse.mly"
                                         (pushPredicate _1)
# 816 "inputparse.ml"
               : 'predicates))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : Ast.Expression.expression) in
    Obj.repr(
# 112 "inputparse.mly"
                      ( Predicate.Atom (_1) )
# 823 "inputparse.ml"
               : Ast.Predicate.predicate))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : Ast.Predicate.predicate) in
    Obj.repr(
# 116 "inputparse.mly"
            ( _1 )
# 830 "inputparse.ml"
               : 'assumption))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'assumption) in
    let _3 = (peek_val parser_env 0 : 'assumption) in
    Obj.repr(
# 117 "inputparse.mly"
                            (Predicate.And [_1;_3])
# 838 "inputparse.ml"
               : 'assumption))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'assumption) in
    let _3 = (peek_val parser_env 0 : 'assumption) in
    Obj.repr(
# 118 "inputparse.mly"
                            (Predicate.Or [_1;_3])
# 846 "inputparse.ml"
               : 'assumption))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'assumption) in
    Obj.repr(
# 119 "inputparse.mly"
                 (Predicate.Not _2)
# 853 "inputparse.ml"
               : 'assumption))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'assumption) in
    Obj.repr(
# 120 "inputparse.mly"
                           (_2)
# 860 "inputparse.ml"
               : 'assumption))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'identifier) in
    Obj.repr(
# 124 "inputparse.mly"
                        ( Expression.Lval (Expression.Symbol (_1)) )
# 867 "inputparse.ml"
               : 'primary_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'constant) in
    Obj.repr(
# 125 "inputparse.mly"
                        ( Expression.Constant (_1) )
# 874 "inputparse.ml"
               : 'primary_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'expression) in
    Obj.repr(
# 126 "inputparse.mly"
                              ( _2  )
# 881 "inputparse.ml"
               : 'primary_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'primary_expression) in
    Obj.repr(
# 130 "inputparse.mly"
                       ( _1 )
# 888 "inputparse.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'postfix_expression) in
    let _3 = (peek_val parser_env 1 : 'expression) in
    Obj.repr(
# 132 "inputparse.mly"
    (  
       Expression.Lval (Expression.Indexed (_1, _3))
    
    )
# 899 "inputparse.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'postfix_expression) in
    Obj.repr(
# 138 "inputparse.mly"
    ( Expression.FunctionCall (_1, []) )
# 906 "inputparse.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'postfix_expression) in
    let _3 = (peek_val parser_env 1 : 'argument_expression_list) in
    Obj.repr(
# 141 "inputparse.mly"
    ( Expression.FunctionCall (_1, _3) )
# 914 "inputparse.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'postfix_expression) in
    let _3 = (peek_val parser_env 0 : 'identifier) in
    Obj.repr(
# 144 "inputparse.mly"
    ( Expression.Lval (Expression.Access(Expression.Dot, _1, _3)) )
# 922 "inputparse.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'postfix_expression) in
    let _3 = (peek_val parser_env 0 : 'identifier) in
    Obj.repr(
# 147 "inputparse.mly"
    ( Expression.Lval (Expression.Access(Expression.Dot, Expression.Lval (Expression.Dereference (_1)), _3)) )
# 930 "inputparse.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'postfix_expression) in
    Obj.repr(
# 150 "inputparse.mly"
    ( Expression.Unary(Expression.PostInc, _1) )
# 937 "inputparse.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'postfix_expression) in
    Obj.repr(
# 153 "inputparse.mly"
            ( Expression.Unary(Expression.PostDec, _1) )
# 944 "inputparse.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 158 "inputparse.mly"
    ( [ _1 ] )
# 951 "inputparse.ml"
               : 'argument_expression_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'argument_expression_list) in
    let _3 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 161 "inputparse.mly"
    ( _1 @ [ _3 ] )
# 959 "inputparse.ml"
               : 'argument_expression_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'postfix_expression) in
    Obj.repr(
# 166 "inputparse.mly"
    ( _1 )
# 966 "inputparse.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'unary_expression) in
    Obj.repr(
# 169 "inputparse.mly"
            ( Expression.Unary(Expression.PreInc, _2) )
# 973 "inputparse.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'unary_expression) in
    Obj.repr(
# 172 "inputparse.mly"
    ( Expression.Unary(Expression.PreDec, _2) )
# 980 "inputparse.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'unary_operator) in
    let _2 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 175 "inputparse.mly"
    ( 
  match _1 with
    Expression.Address -> Expression.addressOf _2
  | _ -> Expression.Unary(_1, _2)
    )
# 992 "inputparse.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 181 "inputparse.mly"
 ( Expression.Lval (Expression.Dereference _2)    )
# 999 "inputparse.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'unary_expression) in
    Obj.repr(
# 183 "inputparse.mly"
    ( failwith "Sizeof with expression not handled" )
# 1006 "inputparse.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _3 = (peek_val parser_env 1 : 'identifier) in
    Obj.repr(
# 186 "inputparse.mly"
    ( failwith "Sizeof not handled" )
# 1013 "inputparse.ml"
               : 'unary_expression))
; (fun parser_env ->
    Obj.repr(
# 190 "inputparse.mly"
                     ( Expression.Address )
# 1019 "inputparse.ml"
               : 'unary_operator))
; (fun parser_env ->
    Obj.repr(
# 191 "inputparse.mly"
                   ( Expression.UnaryPlus )
# 1025 "inputparse.ml"
               : 'unary_operator))
; (fun parser_env ->
    Obj.repr(
# 192 "inputparse.mly"
                    ( Expression.UnaryMinus )
# 1031 "inputparse.ml"
               : 'unary_operator))
; (fun parser_env ->
    Obj.repr(
# 193 "inputparse.mly"
                     ( Expression.BitNot )
# 1037 "inputparse.ml"
               : 'unary_operator))
; (fun parser_env ->
    Obj.repr(
# 194 "inputparse.mly"
                  ( Expression.Not )
# 1043 "inputparse.ml"
               : 'unary_operator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'unary_expression) in
    Obj.repr(
# 198 "inputparse.mly"
                           ( _1 )
# 1050 "inputparse.ml"
               : 'cast_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 2 : 'type_name) in
    let _4 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 200 "inputparse.mly"
            ( Expression.Cast (_2, _4) )
# 1058 "inputparse.ml"
               : 'cast_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 204 "inputparse.mly"
                          ( _1 )
# 1065 "inputparse.ml"
               : 'multiplicative_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'multiplicative_expression) in
    let _3 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 206 "inputparse.mly"
            ( Expression.Binary(Expression.Mul, _1, _3) )
# 1073 "inputparse.ml"
               : 'multiplicative_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'multiplicative_expression) in
    let _3 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 209 "inputparse.mly"
            ( Expression.Binary(Expression.Div, _1, _3) )
# 1081 "inputparse.ml"
               : 'multiplicative_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'multiplicative_expression) in
    let _3 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 212 "inputparse.mly"
            ( Expression.Binary(Expression.Rem, _1, _3) )
# 1089 "inputparse.ml"
               : 'multiplicative_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'multiplicative_expression) in
    Obj.repr(
# 216 "inputparse.mly"
                                    ( _1 )
# 1096 "inputparse.ml"
               : 'additive_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'additive_expression) in
    let _3 = (peek_val parser_env 0 : 'multiplicative_expression) in
    Obj.repr(
# 218 "inputparse.mly"
            ( Expression.Binary(Expression.Plus, _1, _3) )
# 1104 "inputparse.ml"
               : 'additive_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'additive_expression) in
    let _3 = (peek_val parser_env 0 : 'multiplicative_expression) in
    Obj.repr(
# 221 "inputparse.mly"
            ( Expression.Binary(Expression.Minus, _1, _3) )
# 1112 "inputparse.ml"
               : 'additive_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'additive_expression) in
    Obj.repr(
# 225 "inputparse.mly"
                              ( _1 )
# 1119 "inputparse.ml"
               : 'shift_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'shift_expression) in
    let _3 = (peek_val parser_env 0 : 'additive_expression) in
    Obj.repr(
# 227 "inputparse.mly"
     ( Expression.Binary(Expression.LShift, _1, _3) )
# 1127 "inputparse.ml"
               : 'shift_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'shift_expression) in
    let _3 = (peek_val parser_env 0 : 'additive_expression) in
    Obj.repr(
# 230 "inputparse.mly"
     ( Expression.Binary(Expression.RShift, _1, _3) )
# 1135 "inputparse.ml"
               : 'shift_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 234 "inputparse.mly"
                           ( _1 )
# 1142 "inputparse.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'relational_expression) in
    let _3 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 236 "inputparse.mly"
     ( Expression.Binary(Expression.Lt, _1, _3) )
# 1150 "inputparse.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'relational_expression) in
    let _3 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 239 "inputparse.mly"
     ( Expression.Binary(Expression.Gt, _1, _3) )
# 1158 "inputparse.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'relational_expression) in
    let _3 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 242 "inputparse.mly"
     ( Expression.Binary(Expression.Le, _1, _3) )
# 1166 "inputparse.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'relational_expression) in
    let _3 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 245 "inputparse.mly"
     ( Expression.Binary(Expression.Ge, _1, _3) )
# 1174 "inputparse.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'relational_expression) in
    Obj.repr(
# 249 "inputparse.mly"
                                (_1)
# 1181 "inputparse.ml"
               : Ast.Expression.expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : Ast.Expression.expression) in
    let _3 = (peek_val parser_env 0 : 'relational_expression) in
    Obj.repr(
# 251 "inputparse.mly"
            ( Expression.Binary(Expression.Eq, _1, _3) )
# 1189 "inputparse.ml"
               : Ast.Expression.expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : Ast.Expression.expression) in
    let _3 = (peek_val parser_env 0 : 'relational_expression) in
    Obj.repr(
# 254 "inputparse.mly"
            ( Expression.Binary(Expression.Ne, _1, _3) )
# 1197 "inputparse.ml"
               : Ast.Expression.expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : Ast.Expression.expression) in
    Obj.repr(
# 258 "inputparse.mly"
                              ( _1 )
# 1204 "inputparse.ml"
               : 'AND_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'AND_expression) in
    let _3 = (peek_val parser_env 0 : Ast.Expression.expression) in
    Obj.repr(
# 260 "inputparse.mly"
            ( Expression.Binary(Expression.BitAnd, _1, _3) )
# 1212 "inputparse.ml"
               : 'AND_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'AND_expression) in
    Obj.repr(
# 264 "inputparse.mly"
                         ( _1 )
# 1219 "inputparse.ml"
               : 'exclusive_OR_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'exclusive_OR_expression) in
    let _3 = (peek_val parser_env 0 : 'AND_expression) in
    Obj.repr(
# 266 "inputparse.mly"
            ( Expression.Binary (Expression.Xor, _1, _3) )
# 1227 "inputparse.ml"
               : 'exclusive_OR_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'exclusive_OR_expression) in
    Obj.repr(
# 270 "inputparse.mly"
                                  ( _1 )
# 1234 "inputparse.ml"
               : 'inclusive_OR_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'inclusive_OR_expression) in
    let _3 = (peek_val parser_env 0 : 'exclusive_OR_expression) in
    Obj.repr(
# 272 "inputparse.mly"
            ( Expression.Binary (Expression.Or, _1, _3) )
# 1242 "inputparse.ml"
               : 'inclusive_OR_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'inclusive_OR_expression) in
    Obj.repr(
# 276 "inputparse.mly"
                                  ( _1 )
# 1249 "inputparse.ml"
               : 'logical_AND_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'logical_AND_expression) in
    let _3 = (peek_val parser_env 0 : 'inclusive_OR_expression) in
    Obj.repr(
# 278 "inputparse.mly"
            ( Expression.Binary(Expression.And, _1, _3) )
# 1257 "inputparse.ml"
               : 'logical_AND_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'logical_AND_expression) in
    Obj.repr(
# 282 "inputparse.mly"
                                 ( _1 )
# 1264 "inputparse.ml"
               : 'logical_OR_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'logical_OR_expression) in
    let _3 = (peek_val parser_env 0 : 'logical_AND_expression) in
    Obj.repr(
# 284 "inputparse.mly"
            ( Expression.Binary(Expression.Or, _1, _3) )
# 1272 "inputparse.ml"
               : 'logical_OR_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'logical_OR_expression) in
    Obj.repr(
# 288 "inputparse.mly"
                                ( _1 )
# 1279 "inputparse.ml"
               : 'conditional_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 4 : 'logical_OR_expression) in
    let _3 = (peek_val parser_env 2 : 'expression) in
    let _5 = (peek_val parser_env 0 : 'conditional_expression) in
    Obj.repr(
# 290 "inputparse.mly"
            ( failwith "Question mark not handled" )
# 1288 "inputparse.ml"
               : 'conditional_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'conditional_expression) in
    Obj.repr(
# 294 "inputparse.mly"
                                 (_1)
# 1295 "inputparse.ml"
               : 'assignment_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'unary_expression) in
    let _2 = (peek_val parser_env 1 : 'assignment_operator) in
    let _3 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 296 "inputparse.mly"
            (
	  match _1 with
	    Expression.Lval l ->
	      Expression.Assignment (_2, l, _3 )
	  | _ -> failwith "Inputparse : left hand side of assingment not an lvalue" 
	    )
# 1309 "inputparse.ml"
               : 'assignment_expression))
; (fun parser_env ->
    Obj.repr(
# 306 "inputparse.mly"
            ( Expression.Assign )
# 1315 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 309 "inputparse.mly"
            ( Expression.AssignMul )
# 1321 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 312 "inputparse.mly"
            ( Expression.AssignDiv )
# 1327 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 315 "inputparse.mly"
            ( Expression.AssignRem )
# 1333 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 318 "inputparse.mly"
            ( Expression.AssignPlus )
# 1339 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 321 "inputparse.mly"
            ( Expression.AssignMinus )
# 1345 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 324 "inputparse.mly"
            ( Expression.AssignLShift )
# 1351 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 327 "inputparse.mly"
            ( Expression.AssignRShift )
# 1357 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 330 "inputparse.mly"
            ( Expression.AssignBitAnd )
# 1363 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 333 "inputparse.mly"
            ( Expression.AssignXor )
# 1369 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 336 "inputparse.mly"
            ( Expression.AssignBitOr )
# 1375 "inputparse.ml"
               : 'assignment_operator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 341 "inputparse.mly"
            ( _1 )
# 1382 "inputparse.ml"
               : 'expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'expression) in
    let _3 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 343 "inputparse.mly"
            (  failwith "COmma not handled"
	    )
# 1391 "inputparse.ml"
               : 'expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : int) in
    Obj.repr(
# 348 "inputparse.mly"
                    ( Constant.Int (_1) )
# 1398 "inputparse.ml"
               : 'constant))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 353 "inputparse.mly"
             ( _1 )
# 1405 "inputparse.ml"
               : 'identifier))
; (fun parser_env ->
    Obj.repr(
# 356 "inputparse.mly"
               ( [] )
# 1411 "inputparse.ml"
               : 'string_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : string) in
    let _3 = (peek_val parser_env 0 : 'string_list) in
    Obj.repr(
# 357 "inputparse.mly"
                       ( _1 :: _3 )
# 1419 "inputparse.ml"
               : 'string_list))
; (fun parser_env ->
    Obj.repr(
# 360 "inputparse.mly"
        ( (JavaType.Primitive (JavaType.IntType)) )
# 1425 "inputparse.ml"
               : 'type_name))
; (fun parser_env ->
    Obj.repr(
# 361 "inputparse.mly"
          ( (JavaType.Primitive (JavaType.CharType)) )
# 1431 "inputparse.ml"
               : 'type_name))
(* Entry fun_main *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry main *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry env_assumptions *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry lval_list *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
(* Entry modifies_info *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
|]
let yytables =
  { actions=yyact;
    transl_const=yytransl_const;
    transl_block=yytransl_block;
    lhs=yylhs;
    len=yylen;
    defred=yydefred;
    dgoto=yydgoto;
    sindex=yysindex;
    rindex=yyrindex;
    gindex=yygindex;
    tablesize=yytablesize;
    table=yytable;
    check=yycheck;
    error_function=parse_error;
    names_const=yynames_const;
    names_block=yynames_block }
let fun_main (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 1 lexfun lexbuf : ((string * Ast.Predicate.predicate list) list))
let main (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 2 lexfun lexbuf : (Ast.Predicate.predicate list))
let env_assumptions (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 3 lexfun lexbuf : ((Ast.Expression.lval * (Ast.Predicate.predicate)) list))
let lval_list (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 4 lexfun lexbuf : (Ast.Expression.lval list))
let modifies_info (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 5 lexfun lexbuf : ((int * (Ast.Expression.lval list)) list ) * ((string * (Ast.Expression.lval list)) list) * ((Ast.Expression.lval * (string list)) list) * (( (int * int * int * int) * Ast.Expression.lval ) list))
